//
//    Copyright 2004, 2005 Thomas C. McDermott, N5EG
//    This file is part of VNAR - the Vector Network Analyzer program.
//
//    VNAR is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    VNAR is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with VNAR, if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//	  Modified: 05-10-2005	TCM Add routines for Internal Crystal Calibration
//	  Modified: 08-05-2006  TCM Add Error-Checking in case the cal results are poor
//


#pragma once

#include "DisplayRoutines.h"
#include "USB_EZ_interface.h"
#include "Constants.h"
#include <math.h>

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::IO;

#define		NUMCALSTEPS		6		// 6 mandatory calibration steps


namespace VNAR3
{

	/// <summary> 
	/// Summary for InstrumentCal
	///
	/// WARNING: If you change the name of this class, you will need to change the 
	///          'Resource File Name' property for the managed resource compiler tool 
	///          associated with all .resx files this class depends on.  Otherwise,
	///          the designers will not be able to interact properly with localized
	///          resources associated with this form.
	/// </summary>

	/// Instrument Detector Calibration
	public __gc class InstrumentCal : public System::Windows::Forms::Form
	{
	public: 
		InstrumentCal(CalDataSet* CalDS, VNADevice* VNADev, FrequencyGrid* fg, String* StartDir)
		{
			InitializeComponent();
			VNA = VNADev;				///< VNA hardware device
			Cal = CalDS;				///< Calibration Dataset
			StartUpDir = StartDir;		///< Directory where execution started
			FG = fg;					///< Frequency Grid from parent
			Iphase = new int __gc[1024];	///< I-Phase measurements
			Qphase = new int __gc[1024];	///< Q-Phase Measurements
			ReflMagRegression = new int __gc[21, 60]; ///< Shorted Reflection amplitude measurements
			ReflMagPhDir = new int __gc[21, 3];	///< Directivity Reflection raw ADC counts
			ReflMagPhOpen = new int __gc[21, 3];	///< Open Reflection raw ADC counts
			ReflMagPhShort = new int __gc[21, 3];	///< Shorted Reflection raw ADC counts
			ReflDetailMagPhShort = new int __gc[1024, 3];  ///< Shorted Detailed Reflection raw ADC counts
			TranMag = new int __gc[21, 60];  ///< Transmission Amplitude raw ADC counts 
			BufferI = new int __gc[7];		///< holds raw I reading set
			BufferQ = new int __gc[7];		///< holds raw Q reading set
			BufferM = new int __gc[7];		///< holds raw magnitude reading set Lo
			BufferN = new int __gc[7];		///< holds raw magnitude reading set Hi
			BufferP = new int __gc[7];		///< holds raw magnitude reading set Mid

			for(int k=0; k<NUMCALSTEPS; k++) 
				CalStepPass[k] = false;		///< pass/fail for each calibration step
		}
        
	protected: 
		void Dispose(Boolean disposing)
		{
			if (disposing && components)
			{
				components->Dispose();
			}
			__super::Dispose(disposing);
		}
	private: System::Windows::Forms::Label *  label1;
	private: System::Windows::Forms::Button *  OKbutton;
	private: System::Windows::Forms::Button *  Cancelbutton;

	private: System::Windows::Forms::Label *  label2;

	private: System::Windows::Forms::Button *  RxPhaseButton;

	private: System::Windows::Forms::Button *  TxPhaseButton;

	private: System::Windows::Forms::Label *  label4;
	private: System::Windows::Forms::Label *  label5;
	private: System::Windows::Forms::Label *  RxPhaseStat;

	private: System::Windows::Forms::Label *  TxPhaseStat;

	private: System::Windows::Forms::ProgressBar *  progressBar1;
	private: System::Windows::Forms::Label *  label6;
	private: System::Windows::Forms::Label *  label7;
	private: System::Windows::Forms::Button *  TxLowAmpButton;
	private: System::Windows::Forms::Label *  label8;
	private: System::Windows::Forms::Label *  TxLowAmpStat;
	private: System::Windows::Forms::Button *  FrequencyCalButton;
	private: System::Windows::Forms::Label *  label3;
	private: System::Windows::Forms::Label *  FrequencyCalStat;
	private: System::Windows::Forms::TextBox *  MeasFreqTextBox;
	private: System::Windows::Forms::Label *  label9;

	private: System::Windows::Forms::Button *  FreqEnterButton;
	private: System::Windows::Forms::Label *  label10;
	private: System::Windows::Forms::Label *  label11;
	private: System::Windows::Forms::Label *  label12;

	private: System::Windows::Forms::Button *  DirectivityCalButton;
	private: System::Windows::Forms::Label *  DirectivityCalStat;
	private: System::Windows::Forms::Label *  label14;

	private: System::Windows::Forms::Button *  RxAmpButtonOpen;
	private: System::Windows::Forms::Label *  RxAmpStatShort;
	private: System::Windows::Forms::Label *  label15;
	private: System::Windows::Forms::Label *  RxAmpStatOpen;
	private: System::Windows::Forms::Button *  RxAmpButtonShort;
	private: System::Windows::Forms::Label *  ICO1Label;
	private: System::Windows::Forms::Label *  ICO2Label;
	private: System::Windows::Forms::GroupBox *  Optional;

	private: System::ComponentModel::IContainer *  components;

	private:
		/// <summary>
		/// Required designer variable.
		/// </summary>


		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		void InitializeComponent(void)
		{
			System::Resources::ResourceManager *  resources = new System::Resources::ResourceManager(__typeof(VNAR3::InstrumentCal));
			this->label1 = new System::Windows::Forms::Label();
			this->OKbutton = new System::Windows::Forms::Button();
			this->Cancelbutton = new System::Windows::Forms::Button();
			this->RxPhaseButton = new System::Windows::Forms::Button();
			this->label2 = new System::Windows::Forms::Label();
			this->TxPhaseButton = new System::Windows::Forms::Button();
			this->label4 = new System::Windows::Forms::Label();
			this->label5 = new System::Windows::Forms::Label();
			this->RxPhaseStat = new System::Windows::Forms::Label();
			this->RxAmpStatShort = new System::Windows::Forms::Label();
			this->TxPhaseStat = new System::Windows::Forms::Label();
			this->progressBar1 = new System::Windows::Forms::ProgressBar();
			this->label6 = new System::Windows::Forms::Label();
			this->label7 = new System::Windows::Forms::Label();
			this->TxLowAmpButton = new System::Windows::Forms::Button();
			this->label8 = new System::Windows::Forms::Label();
			this->TxLowAmpStat = new System::Windows::Forms::Label();
			this->FrequencyCalButton = new System::Windows::Forms::Button();
			this->label3 = new System::Windows::Forms::Label();
			this->FrequencyCalStat = new System::Windows::Forms::Label();
			this->MeasFreqTextBox = new System::Windows::Forms::TextBox();
			this->label9 = new System::Windows::Forms::Label();
			this->FreqEnterButton = new System::Windows::Forms::Button();
			this->label10 = new System::Windows::Forms::Label();
			this->label11 = new System::Windows::Forms::Label();
			this->label12 = new System::Windows::Forms::Label();
			this->DirectivityCalButton = new System::Windows::Forms::Button();
			this->DirectivityCalStat = new System::Windows::Forms::Label();
			this->label14 = new System::Windows::Forms::Label();
			this->RxAmpButtonOpen = new System::Windows::Forms::Button();
			this->label15 = new System::Windows::Forms::Label();
			this->RxAmpStatOpen = new System::Windows::Forms::Label();
			this->RxAmpButtonShort = new System::Windows::Forms::Button();
			this->ICO1Label = new System::Windows::Forms::Label();
			this->ICO2Label = new System::Windows::Forms::Label();
			this->Optional = new System::Windows::Forms::GroupBox();
			this->SuspendLayout();
			// 
			// label1
			// 
			this->label1->Font = new System::Drawing::Font(S"Verdana", 15.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->label1->Location = System::Drawing::Point(144, 6);
			this->label1->Name = S"label1";
			this->label1->Size = System::Drawing::Size(426, 29);
			this->label1->TabIndex = 0;
			this->label1->Text = S"Detector Calibration";
			this->label1->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			// 
			// OKbutton
			// 
			this->OKbutton->Location = System::Drawing::Point(456, 488);
			this->OKbutton->Name = S"OKbutton";
			this->OKbutton->Size = System::Drawing::Size(101, 30);
			this->OKbutton->TabIndex = 1;
			this->OKbutton->Text = S"Save";
			this->OKbutton->Click += new System::EventHandler(this, OKbutton_Click);
			// 
			// Cancelbutton
			// 
			this->Cancelbutton->DialogResult = System::Windows::Forms::DialogResult::Cancel;
			this->Cancelbutton->Location = System::Drawing::Point(584, 488);
			this->Cancelbutton->Name = S"Cancelbutton";
			this->Cancelbutton->Size = System::Drawing::Size(105, 28);
			this->Cancelbutton->TabIndex = 2;
			this->Cancelbutton->Text = S"Cancel";
			this->Cancelbutton->Click += new System::EventHandler(this, Cancelbutton_Click);
			// 
			// RxPhaseButton
			// 
			this->RxPhaseButton->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->RxPhaseButton->Location = System::Drawing::Point(520, 64);
			this->RxPhaseButton->Name = S"RxPhaseButton";
			this->RxPhaseButton->Size = System::Drawing::Size(176, 48);
			this->RxPhaseButton->TabIndex = 3;
			this->RxPhaseButton->Text = S"Refl Phase Detector Calibration";
			this->RxPhaseButton->Click += new System::EventHandler(this, RxPhaseButton_Click);
			// 
			// label2
			// 
			this->label2->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->label2->Location = System::Drawing::Point(96, 64);
			this->label2->Name = S"label2";
			this->label2->Size = System::Drawing::Size(414, 48);
			this->label2->TabIndex = 5;
			this->label2->Text = S"Connect approximately one meter long 50-ohm cable, shorted at far end, to Tx conn" 
				S"ector.";
			// 
			// TxPhaseButton
			// 
			this->TxPhaseButton->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->TxPhaseButton->Location = System::Drawing::Point(520, 208);
			this->TxPhaseButton->Name = S"TxPhaseButton";
			this->TxPhaseButton->Size = System::Drawing::Size(176, 48);
			this->TxPhaseButton->TabIndex = 6;
			this->TxPhaseButton->Text = S"Trans Phase Detector Calibration";
			this->TxPhaseButton->Click += new System::EventHandler(this, TxPhaseButton_Click);
			// 
			// label4
			// 
			this->label4->Location = System::Drawing::Point(152, 32);
			this->label4->Name = S"label4";
			this->label4->Size = System::Drawing::Size(415, 28);
			this->label4->TabIndex = 9;
			this->label4->Text = S"(This calibration normally needs to be run only one time.)";
			this->label4->TextAlign = System::Drawing::ContentAlignment::MiddleCenter;
			// 
			// label5
			// 
			this->label5->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Underline, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->label5->Location = System::Drawing::Point(16, 38);
			this->label5->Name = S"label5";
			this->label5->Size = System::Drawing::Size(67, 20);
			this->label5->TabIndex = 10;
			this->label5->Text = S"Status";
			// 
			// RxPhaseStat
			// 
			this->RxPhaseStat->Location = System::Drawing::Point(16, 56);
			this->RxPhaseStat->Name = S"RxPhaseStat";
			this->RxPhaseStat->Size = System::Drawing::Size(48, 48);
			this->RxPhaseStat->TabIndex = 11;
			this->RxPhaseStat->Visible = false;
			// 
			// RxAmpStatShort
			// 
			this->RxAmpStatShort->Location = System::Drawing::Point(16, 104);
			this->RxAmpStatShort->Name = S"RxAmpStatShort";
			this->RxAmpStatShort->Size = System::Drawing::Size(48, 48);
			this->RxAmpStatShort->TabIndex = 12;
			this->RxAmpStatShort->Visible = false;
			// 
			// TxPhaseStat
			// 
			this->TxPhaseStat->Location = System::Drawing::Point(16, 200);
			this->TxPhaseStat->Name = S"TxPhaseStat";
			this->TxPhaseStat->Size = System::Drawing::Size(48, 48);
			this->TxPhaseStat->TabIndex = 13;
			this->TxPhaseStat->Visible = false;
			// 
			// progressBar1
			// 
			this->progressBar1->Location = System::Drawing::Point(32, 520);
			this->progressBar1->Maximum = 1024;
			this->progressBar1->Name = S"progressBar1";
			this->progressBar1->Size = System::Drawing::Size(664, 13);
			this->progressBar1->TabIndex = 15;
			// 
			// label6
			// 
			this->label6->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->label6->Location = System::Drawing::Point(96, 208);
			this->label6->Name = S"label6";
			this->label6->Size = System::Drawing::Size(408, 56);
			this->label6->TabIndex = 16;
			this->label6->Text = S"Connect approximately three meter long (plus or minus one-half meter) 50-ohm cabl" 
				S"e and 40 dB attenuator from Rx to Tx.";
			// 
			// label7
			// 
			this->label7->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->label7->Location = System::Drawing::Point(96, 112);
			this->label7->Name = S"label7";
			this->label7->Size = System::Drawing::Size(414, 48);
			this->label7->TabIndex = 17;
			this->label7->Text = S"Connect approximately one meter long 50-ohm cable, shorted at far end, to Tx conn" 
				S"ector.";
			// 
			// TxLowAmpButton
			// 
			this->TxLowAmpButton->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->TxLowAmpButton->Location = System::Drawing::Point(520, 256);
			this->TxLowAmpButton->Name = S"TxLowAmpButton";
			this->TxLowAmpButton->Size = System::Drawing::Size(176, 48);
			this->TxLowAmpButton->TabIndex = 18;
			this->TxLowAmpButton->Text = S"Trans Amplitude Detector Calibration";
			this->TxLowAmpButton->Click += new System::EventHandler(this, TxLowAmpButton_Click);
			// 
			// label8
			// 
			this->label8->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->label8->Location = System::Drawing::Point(96, 264);
			this->label8->Name = S"label8";
			this->label8->Size = System::Drawing::Size(408, 45);
			this->label8->TabIndex = 19;
			this->label8->Text = S"Connect 40 dB attenuator (50 ohm type) from Rx to Tx. (May use three-meter cable " 
				S"and setup from previous step).";
			// 
			// TxLowAmpStat
			// 
			this->TxLowAmpStat->Location = System::Drawing::Point(16, 256);
			this->TxLowAmpStat->Name = S"TxLowAmpStat";
			this->TxLowAmpStat->Size = System::Drawing::Size(48, 48);
			this->TxLowAmpStat->TabIndex = 20;
			this->TxLowAmpStat->Visible = false;
			// 
			// FrequencyCalButton
			// 
			this->FrequencyCalButton->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->FrequencyCalButton->Location = System::Drawing::Point(512, 384);
			this->FrequencyCalButton->Name = S"FrequencyCalButton";
			this->FrequencyCalButton->Size = System::Drawing::Size(176, 48);
			this->FrequencyCalButton->TabIndex = 21;
			this->FrequencyCalButton->Text = S"Internal Crystal Frequency Calibration";
			this->FrequencyCalButton->Click += new System::EventHandler(this, FrequencyCalButton_Click);
			// 
			// label3
			// 
			this->label3->Location = System::Drawing::Point(96, 368);
			this->label3->Name = S"label3";
			this->label3->Size = System::Drawing::Size(408, 24);
			this->label3->TabIndex = 22;
			this->label3->Text = S"1. Connect frequency counter to TX.";
			// 
			// FrequencyCalStat
			// 
			this->FrequencyCalStat->Location = System::Drawing::Point(16, 384);
			this->FrequencyCalStat->Name = S"FrequencyCalStat";
			this->FrequencyCalStat->Size = System::Drawing::Size(56, 49);
			this->FrequencyCalStat->TabIndex = 23;
			this->FrequencyCalStat->Visible = false;
			// 
			// MeasFreqTextBox
			// 
			this->MeasFreqTextBox->Location = System::Drawing::Point(256, 456);
			this->MeasFreqTextBox->Name = S"MeasFreqTextBox";
			this->MeasFreqTextBox->Size = System::Drawing::Size(111, 23);
			this->MeasFreqTextBox->TabIndex = 24;
			this->MeasFreqTextBox->Text = S"100000000";
			// 
			// label9
			// 
			this->label9->Location = System::Drawing::Point(96, 456);
			this->label9->Name = S"label9";
			this->label9->Size = System::Drawing::Size(153, 23);
			this->label9->TabIndex = 25;
			this->label9->Text = S"Measured Frequency";
			// 
			// FreqEnterButton
			// 
			this->FreqEnterButton->Location = System::Drawing::Point(376, 456);
			this->FreqEnterButton->Name = S"FreqEnterButton";
			this->FreqEnterButton->Size = System::Drawing::Size(56, 24);
			this->FreqEnterButton->TabIndex = 26;
			this->FreqEnterButton->Text = S"Enter";
			this->FreqEnterButton->Click += new System::EventHandler(this, FreqEnterButton_Click);
			// 
			// label10
			// 
			this->label10->Location = System::Drawing::Point(96, 384);
			this->label10->Name = S"label10";
			this->label10->Size = System::Drawing::Size(408, 16);
			this->label10->TabIndex = 27;
			this->label10->Text = S"2. Click \"Internal Crystal Frequency Calibration\" button.";
			// 
			// label11
			// 
			this->label11->Location = System::Drawing::Point(96, 408);
			this->label11->Name = S"label11";
			this->label11->Size = System::Drawing::Size(408, 32);
			this->label11->TabIndex = 28;
			this->label11->Text = S"3. Measure and enter frequency to nearest hertz (approximately 100 MHz).";
			// 
			// label12
			// 
			this->label12->Location = System::Drawing::Point(96, 440);
			this->label12->Name = S"label12";
			this->label12->Size = System::Drawing::Size(408, 16);
			this->label12->TabIndex = 29;
			this->label12->Text = S"4. Click \"Enter\" button.";
			// 
			// DirectivityCalButton
			// 
			this->DirectivityCalButton->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->DirectivityCalButton->Location = System::Drawing::Point(520, 304);
			this->DirectivityCalButton->Name = S"DirectivityCalButton";
			this->DirectivityCalButton->Size = System::Drawing::Size(176, 48);
			this->DirectivityCalButton->TabIndex = 31;
			this->DirectivityCalButton->Text = S"Coupler Directivity Calibration";
			this->DirectivityCalButton->Click += new System::EventHandler(this, DirectivityCalButton_Click);
			// 
			// DirectivityCalStat
			// 
			this->DirectivityCalStat->Location = System::Drawing::Point(16, 304);
			this->DirectivityCalStat->Name = S"DirectivityCalStat";
			this->DirectivityCalStat->Size = System::Drawing::Size(48, 48);
			this->DirectivityCalStat->TabIndex = 32;
			this->DirectivityCalStat->Visible = false;
			// 
			// label14
			// 
			this->label14->Location = System::Drawing::Point(96, 312);
			this->label14->Name = S"label14";
			this->label14->Size = System::Drawing::Size(408, 32);
			this->label14->TabIndex = 33;
			this->label14->Text = S"Connect 40 dB attenuator directly to the TX connector (no cable).";
			// 
			// RxAmpButtonOpen
			// 
			this->RxAmpButtonOpen->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->RxAmpButtonOpen->Location = System::Drawing::Point(520, 160);
			this->RxAmpButtonOpen->Name = S"RxAmpButtonOpen";
			this->RxAmpButtonOpen->Size = System::Drawing::Size(176, 48);
			this->RxAmpButtonOpen->TabIndex = 34;
			this->RxAmpButtonOpen->Text = S"Refl Amplitude Detector Cal Open";
			this->RxAmpButtonOpen->Click += new System::EventHandler(this, RxAmpButtonOpen_Click);
			// 
			// label15
			// 
			this->label15->Location = System::Drawing::Point(96, 160);
			this->label15->Name = S"label15";
			this->label15->Size = System::Drawing::Size(408, 48);
			this->label15->TabIndex = 35;
			this->label15->Text = S"Connect approximately one meter long 50-ohm cable, open at far end, to Tx connect" 
				S"or.";
			// 
			// RxAmpStatOpen
			// 
			this->RxAmpStatOpen->Location = System::Drawing::Point(16, 152);
			this->RxAmpStatOpen->Name = S"RxAmpStatOpen";
			this->RxAmpStatOpen->Size = System::Drawing::Size(48, 48);
			this->RxAmpStatOpen->TabIndex = 36;
			this->RxAmpStatOpen->Visible = false;
			// 
			// RxAmpButtonShort
			// 
			this->RxAmpButtonShort->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Bold, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->RxAmpButtonShort->Location = System::Drawing::Point(520, 112);
			this->RxAmpButtonShort->Name = S"RxAmpButtonShort";
			this->RxAmpButtonShort->Size = System::Drawing::Size(176, 48);
			this->RxAmpButtonShort->TabIndex = 37;
			this->RxAmpButtonShort->Text = S"Refl Amplitude Detector Cal Short";
			this->RxAmpButtonShort->Click += new System::EventHandler(this, RxAmpButtonShort_Click);
			// 
			// ICO1Label
			// 
			this->ICO1Label->Image = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"ICO1Label.Image")));
			this->ICO1Label->Location = System::Drawing::Point(19, 448);
			this->ICO1Label->Name = S"ICO1Label";
			this->ICO1Label->Size = System::Drawing::Size(37, 25);
			this->ICO1Label->TabIndex = 38;
			this->ICO1Label->Visible = false;
			// 
			// ICO2Label
			// 
			this->ICO2Label->Image = (__try_cast<System::Drawing::Image *  >(resources->GetObject(S"ICO2Label.Image")));
			this->ICO2Label->Location = System::Drawing::Point(25, 480);
			this->ICO2Label->Name = S"ICO2Label";
			this->ICO2Label->Size = System::Drawing::Size(25, 25);
			this->ICO2Label->TabIndex = 39;
			this->ICO2Label->Visible = false;
			// 
			// Optional
			// 
			this->Optional->BackColor = System::Drawing::Color::Transparent;
			this->Optional->Location = System::Drawing::Point(87, 352);
			this->Optional->Name = S"Optional";
			this->Optional->Size = System::Drawing::Size(609, 134);
			this->Optional->TabIndex = 40;
			this->Optional->TabStop = false;
			this->Optional->Text = S"Optional";
			// 
			// InstrumentCal
			// 
			this->AcceptButton = this->OKbutton;
			this->AutoScale = false;
			this->AutoScaleBaseSize = System::Drawing::Size(7, 16);
			this->AutoScroll = true;
			this->CancelButton = this->Cancelbutton;
			this->ClientSize = System::Drawing::Size(736, 542);
			this->Controls->Add(this->ICO2Label);
			this->Controls->Add(this->ICO1Label);
			this->Controls->Add(this->RxAmpButtonShort);
			this->Controls->Add(this->RxAmpStatOpen);
			this->Controls->Add(this->label15);
			this->Controls->Add(this->RxAmpButtonOpen);
			this->Controls->Add(this->label14);
			this->Controls->Add(this->DirectivityCalStat);
			this->Controls->Add(this->DirectivityCalButton);
			this->Controls->Add(this->label12);
			this->Controls->Add(this->label11);
			this->Controls->Add(this->label10);
			this->Controls->Add(this->FreqEnterButton);
			this->Controls->Add(this->label9);
			this->Controls->Add(this->MeasFreqTextBox);
			this->Controls->Add(this->FrequencyCalStat);
			this->Controls->Add(this->label3);
			this->Controls->Add(this->FrequencyCalButton);
			this->Controls->Add(this->TxLowAmpStat);
			this->Controls->Add(this->label8);
			this->Controls->Add(this->TxLowAmpButton);
			this->Controls->Add(this->label7);
			this->Controls->Add(this->label6);
			this->Controls->Add(this->progressBar1);
			this->Controls->Add(this->TxPhaseStat);
			this->Controls->Add(this->RxAmpStatShort);
			this->Controls->Add(this->RxPhaseStat);
			this->Controls->Add(this->label5);
			this->Controls->Add(this->label4);
			this->Controls->Add(this->TxPhaseButton);
			this->Controls->Add(this->label2);
			this->Controls->Add(this->RxPhaseButton);
			this->Controls->Add(this->Cancelbutton);
			this->Controls->Add(this->OKbutton);
			this->Controls->Add(this->label1);
			this->Controls->Add(this->Optional);
			this->Font = new System::Drawing::Font(S"Verdana", 9.75F, System::Drawing::FontStyle::Regular, System::Drawing::GraphicsUnit::Point, (System::Byte)0);
			this->Location = System::Drawing::Point(128, 239);
			this->Name = S"InstrumentCal";
			this->Text = S"Detector Cal";
			this->ResumeLayout(false);

		}		

private: VNADevice* VNA;					///< Vector Network Analyzer hardware object
private: CalDataSet* Cal;					///< Calibration Data Set
private: FrequencyGrid* FG;					///< Our frequency grid
private: String* StartUpDir;				///< Startup directory
private: int Iphase __gc[], Qphase __gc[];	///< hold phase detector results from sweep
private: int ReflMagRegression __gc[ , ];	///< Reflection Magnitude vs level sweep
private: int ReflMagPhDir __gc[ , ];		///< Directivity Magnitude & Phase data from reflection sweep
private: int ReflMagPhOpen __gc[ , ];		///< Open Magnitude & Phase data  rom reflection sweep
private: int ReflMagPhShort __gc[ , ];		///< Shorted Magnitude & Phase data from reflection sweep
private: int ReflDetailMagPhShort __gc[ , ]; ///< Shorted Detailed Mag & Phase data from reflection sweep
private: int TranMag __gc[ , ];				///< Transmission magnitude detector vs level sweep
private: int BufferI __gc[], BufferQ __gc[];
private: int BufferN __gc[], BufferM __gc[]; ///< Temporary raw readings
private: int BufferP __gc[];
private: int MeasFrequency;					///< Measured Frequency from crystal calibration
private: bool CalStepPass __nogc[NUMCALSTEPS];		///< Array of Calibrations Steps. TRUE = step passed

		 /// Reflection Phase Detector Calibration button click handler
private: System::Void RxPhaseButton_Click(System::Object *  sender, System::EventArgs *  e)
		 {

			 // Reflection Phase Detector Calibration

			VNA_RXBUFFER *RxBuf = new VNA_RXBUFFER;
			VNA_TXBUFFER *TxBuf = new VNA_TXBUFFER;
			int Fdesired;

			progressBar1->Value = 0;
	

			// run a sweep of 1024 points, calibrating Rx phase detector
			for (long i=0; i<1024; i++)
			{
				// Compute spot frequency

				Fdesired = Convert::ToInt32(200000.0+( i*(120000000.0-200000.0)/1024.0));

				TxBuf->TxAccum = FG->DDS(Fdesired);
				TxBuf->IDAClevelHi = MAX_DDS_LEVEL;		// Max transmit level
				TxBuf->ReplyType = 0;
				TxBuf->MeasureDelay = 0;
				TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level

				// Take 7 readings at the same frequency

				for(int k=0; k<7; k++)
				{
                    VNA->WriteRead(TxBuf, RxBuf);
					BufferI[k] = RxBuf->ReflPI;
					BufferQ[k] = RxBuf->ReflPQ;
				}

				Iphase[i] = Median7i(BufferI);	// use median filter to smooth raw value
				Qphase[i] = Median7i(BufferQ);

				progressBar1->Value = i;
				if(i%20 == 0)
					progressBar1->Update();
			}

			if(Cal->RxDet->PhaseCal(Iphase, Qphase))		// TRUE if cal sucessful
			{
				RxPhaseStat->Image = ICO1Label->Image;				// checkmark
				CalStepPass[0] = true;
			}
			else
			{
				RxPhaseStat->Image = ICO2Label->Image;				// 'x'
				CalStepPass[0] = false;
				ShowCalStepFailMessage();
			}

			RxPhaseStat->Visible = true;
		 }
		 /// Transmission Phase Detector Calibration button click handler
private: System::Void TxPhaseButton_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 // Transmission Phase Detector Calibration

			VNA_RXBUFFER *RxBuf = new VNA_RXBUFFER;
			VNA_TXBUFFER *TxBuf = new VNA_TXBUFFER;
			int Fdesired;

			progressBar1->Value = 0;

			// run a sweep of ~1024 points, calibrating Tx phase detector
			// skip really low frequencies and really high frequencies
			for (long i=0; i<1024; i++)
			{
				// Compute spot frequency

				Fdesired = Convert::ToInt32(200000.0+( i*(120000000.0-200000.0)/1024.0));

				TxBuf->TxAccum = FG->DDS(Fdesired);	
				TxBuf->IDAClevelHi = MAX_DDS_LEVEL;		// Max transmit level
				TxBuf->ReplyType = 0;
				TxBuf->MeasureDelay = 0;
				TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level

				// Take 7 readings at the same frequency
				for(int k=0; k<7; k++)
				{
                    VNA->WriteRead(TxBuf, RxBuf);
					BufferI[k] = RxBuf->TranPI;
					BufferQ[k] = RxBuf->TranPQ;
				}

				Iphase[i] = Median7i(BufferI);		// use median filter to smooth raw reading
				Qphase[i] = Median7i(BufferQ);

				progressBar1->Value = i;
				if(i%20 == 0)
					progressBar1->Update();
			}
		
			if(Cal->TxDet->PhaseCal(Iphase, Qphase))	// TRUE if phase cal was sucessful
			{
				TxPhaseStat->Image = ICO1Label->Image;				// checkmark
				CalStepPass[1] = true;
			}
			else
			{
				TxPhaseStat->Image = ICO2Label->Image;				// 'x'
				CalStepPass[1] = false;
				ShowCalStepFailMessage();
			}

			TxPhaseStat->Visible = true;
		 }

		 /// OK button click handler
private: System::Void OKbutton_Click(System::Object *  sender, System::EventArgs *  e)
		 {

			double mag, phase;
			double& rmag = mag;
			double& rphase = phase;
			double b1, b2;
			int Fdesired;
			int FreqIdx;

			 // Check that all relevant groups of data have been run, else
			 // display error message and give chance to rerun missing sets.

			for (int k = 0; k<NUMCALSTEPS; k++)
				if(CalStepPass[k] = false)
			{
				MessageBox::Show(S"Some calibration steps not completed or failed."
					S"  Be sure to complete all required steps before finishing."
					S"\n\rClose this window, then 'Cancel' if you do not wish to save Detector Cal data.",
					S"Error: Missing Steps", MessageBoxButtons::OK, MessageBoxIcon::Error);
				return;
			}


			// Normalize 'b'  in y = mx + b   for (ShortDB + OpenDB) / 2

			for (FreqIdx=0; FreqIdx<21; FreqIdx++)
			{

				// Compute spot frequency

				if (FreqIdx < 9)
					Fdesired = Convert::ToInt32((FreqIdx + 2) * 100000);	// every 100 KHz starting at 200 khz.
				else
					Fdesired = Convert::ToInt32((FreqIdx-8) * 10000000);	// every ten MHz.


                Cal->ResolveReflPolar(ReflMagPhOpen[FreqIdx, 1], ReflMagPhOpen[FreqIdx, 2],
					ReflMagPhOpen[FreqIdx, 0], Fdesired, rmag, rphase, false);
				b1 = 20.0 * log10(mag);	// convert to dB

                Cal->ResolveReflPolar(ReflMagPhShort[FreqIdx, 1], ReflMagPhShort[FreqIdx, 2],
					ReflMagPhShort[FreqIdx, 0], Fdesired, rmag, rphase, false);
				b2 = 20.0 * log10(mag);	// convert to dB

				Cal->RxDet->b[FreqIdx] -= (b1+b2)/2.0;		// 'b' value is halfway between open and short values
			}

			// Derive phase-angle dependent directional coupler error.  The model is a sine-wave error
			// with two terms: magnitude and phase-offset. Optimization criteria minimizes MSE
			// from the 1024-point Detector Shorted Calibration run by varying angle-offset and magnitude.
			// 0 db is the zero value for the MSE minimizer.
			// ReflDetailMagPhShort[,] contains the raw ADC readings from a 1024 point sweep

			double SquaredError = 1000000;	// 'large' error
			int PhaseEstimate;				// best estimate for models's phase offset
			double MagEstimate = 0.5;		// initial estimate for model's magnitude
			int angle;
			double error;
			double temp;
#define BEGINF	50							// Start MSE summation at about 5 MHz
#define ENDF	850							// Complete MSE summation at about 90 MHz

			// Search for phase-angle offset giving smallest MSE

			for (int offset = -180; offset< +180; offset++)	// Check over full range
			{
				// Compute Sum Squared Error

				error = 0;
				for(int j=BEGINF; j<ENDF; j++)
				{

					Fdesired = Convert::ToInt32(200000.0+( j*(120000000.0-200000.0)/1024.0));
					Cal->ResolveReflPolar(ReflDetailMagPhShort[j, 1], ReflDetailMagPhShort[j, 2],
						ReflDetailMagPhShort[j, 0], Fdesired, rmag, rphase, false);

					angle = (int)(phase - (double)offset);
					if (angle > 180)
						angle -= 360;
					if (angle < -180)
						angle += 360;

					temp = (20.0 * log10(mag)) - MagEstimate * sin(angle * DEGR2RAD);
					error += (temp * temp);
				}
				if (error < SquaredError)	// Save if it's the minimum (MSE)
				{
					SquaredError = error;
					PhaseEstimate = offset;
				}
			}

			// Search for magnitude giving smallest MSE at the just-determined offset angle

			SquaredError = 1000000;	// 'large' error
			for (double magnitude = 0.0; magnitude<1.2; magnitude += 0.01)	// Check over 0 to 1.2 dB
			{
				// Compute Sum Squared Error

				error = 0;
				for(int j=BEGINF; j<ENDF; j++)
				{

					Fdesired = Convert::ToInt32(200000.0+( j*(120000000.0-200000.0)/1024.0));
					Cal->ResolveReflPolar(ReflDetailMagPhShort[j, 1], ReflDetailMagPhShort[j, 2],
						ReflDetailMagPhShort[j, 0], Fdesired, rmag, rphase, false);

					angle = (int)(phase - (double)PhaseEstimate);
					if (angle > 180)
						angle -= 360;
					if (angle < -180)
						angle += 360;

					temp = (20.0 * log10(mag)) - magnitude * sin(angle * DEGR2RAD);
					error += (temp * temp);
				}
				if (error < SquaredError)	// Save if it's the minimum (MSE)
				{
					SquaredError = error;
					MagEstimate = magnitude;
				}
			}

			// Re-compute phase search with improved value of MagEstimate

			SquaredError = 1000000;	// 'large' error
			for (int offset = -180; offset< +180; offset++)	// Check over full range

				// BUGBUG - narrow the phase search range the second time around?

			{
				// Compute Sum Squared Error

				error = 0;
				for(int j=BEGINF; j<ENDF; j++)
				{

					Fdesired = Convert::ToInt32(200000.0+( j*(120000000.0-200000.0)/1024.0));
					Cal->ResolveReflPolar(ReflDetailMagPhShort[j, 1], ReflDetailMagPhShort[j, 2],
						ReflDetailMagPhShort[j, 0], Fdesired, rmag, rphase, false);

					angle = (int)(phase - (double)offset);
					if (angle > 180)
						angle -= 360;
					if (angle < -180)
						angle += 360;

					temp = (20.0 * log10(mag)) - MagEstimate * sin(angle * DEGR2RAD);
					error += (temp * temp);
				}
				if (error < SquaredError)	// Save if it's the minimum (MSE)
				{
					SquaredError = error;
					PhaseEstimate = offset;
				}
			}

			Cal->RxDet->CouplerPhase = PhaseEstimate;
			Cal->RxDet->CouplerMagnitude = MagEstimate;


			// Save Instrument Calibration values to detector.ica

			FileStream* fs;
			BinaryWriter* bw;

			String * filename = String::Concat(StartUpDir, S"\\detector.ica");
			try
			{
				// Create a filestream & binary writer

				fs = new FileStream(filename, FileMode::Create, FileAccess::Write);
				bw = new BinaryWriter(fs);
			}
			catch(System::Exception* pe)
			{
				MessageBox::Show(S"Error. Could not Write Detector Calibration file.", pe->Message,
					MessageBoxButtons::OK, MessageBoxIcon::Error);
				return;
			}

			// Define header to match file identifying type and version

			String* recognized = new String(S"VNA Detector Calibration Data Set Version 2.0.2");
			bw->Write(recognized);		// put string header on outfile

			// write the AD8302 Phase Detector constants and error tables
			bw->Write(Cal->RxDet->centerPhase);
			bw->Write(Cal->RxDet->lowerPhase);
			bw->Write(Cal->RxDet->upperPhase);
			for (int degree = 0; degree<360; degree++)
				bw->Write(Cal->RxDet->pherror[degree]);

			bw->Write(Cal->TxDet->centerPhase);
			bw->Write(Cal->TxDet->lowerPhase);
			bw->Write(Cal->TxDet->upperPhase);
			for (int degree = 0; degree<360; degree++)
				bw->Write(Cal->TxDet->pherror[degree]);

			// write the AD8302 Amplitude Detector tables

			for (int i=0; i<21; i++)
			{
                bw->Write(Cal->RxDet->m[i]);
                bw->Write(Cal->RxDet->b[i]);
                bw->Write(Cal->RxDet->r[i]);
				bw->Write(Cal->RxDet->flat[i]);
			}
			
			for (int i=0; i<21; i++)
			{
                bw->Write(Cal->TxDet->m[i]);
                bw->Write(Cal->TxDet->b[i]);
                bw->Write(Cal->TxDet->r[i]);
				bw->Write(Cal->TxDet->flat[i]);
			}

			// write the frequency error (or zero if none entered)

			bw->Write(FG->Ferror);

			// Write the Directivity Calibration raw values 10-18-2005

			for (int i=0; i<21; i++)
			{
				bw->Write(Cal->RxDet->DirIphs[i]);
				bw->Write(Cal->RxDet->DirQphs[i]);
				bw->Write(Cal->RxDet->DirMag[i]);
			}

			// Write the Directional Coupler error terms 12-26-2005

			bw->Write(Cal->RxDet->CouplerPhase);
			bw->Write(Cal->RxDet->CouplerMagnitude);

			fs->Flush();
			fs->Close();

			DialogResult = DialogResult::OK;		// Close the form
		 }


		 /// Transmission Amplitude Detector Calibration button click handler
private: System::Void TxLowAmpButton_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			
			// Transmission Amplitude detector Calibration - with 40 dB attenuator in path
			 
			VNA_RXBUFFER *RxBuf = new VNA_RXBUFFER;
			VNA_TXBUFFER *TxBuf = new VNA_TXBUFFER;
			int Fdesired;

			TxBuf->ReplyType = 0;
			TxBuf->MeasureDelay = 0;		    // No delay
			TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level

			progressBar1->Value = 0;

			// run a sweep of 21 frequencies, calibrating Trans Amplitude detector
			for (long FreqIdx=0; FreqIdx<21; FreqIdx++)
			{

				// Compute spot frequency

				if (FreqIdx < 9)
					Fdesired = Convert::ToInt32((FreqIdx + 2) * 100000);	// every 100 KHz starting at 200 khz.
				else
					Fdesired = Convert::ToInt32((FreqIdx-8) * 10000000);	// every ten MHz.

				TxBuf->TxAccum = FG->DDS(Fdesired);

				for (int level=0; level<30; level++)	// Measure over 60 dB range
				{
 					TxBuf->IDAClevelHi = TxLevLinear(-level);	// 0 to -29 dbm
					TxBuf->IDAClevelLo = TxLevLinear(-level-30);	// -30 to -59 dbm

					for(int k=0; k<7; k++)				// Take 7 readings at each point
					{
						VNA->WriteRead(TxBuf, RxBuf);
						BufferM[k] = RxBuf->TranMQHi;
						BufferN[k] = RxBuf->TranMQLo;
					}
					TranMag[FreqIdx, level] = Median7i(BufferM);		// Filter the readings
					TranMag[FreqIdx, level+30] = Median7i(BufferN);

				}
				progressBar1->Value = FreqIdx * progressBar1->Maximum/20;
				progressBar1->Update();
			}
			if(Cal->TxDet->AmpCal(TranMag))		// AmpCal forms y = mx + b for transmission detector
			{
                CalStepPass[2] = true;
				TxLowAmpStat->Image = ICO1Label->Image;
			}
			else
			{
                CalStepPass[2] = false;
				TxLowAmpStat->Image = ICO2Label->Image;
				ShowCalStepFailMessage();
			}

			TxLowAmpStat->Visible = true;

		 }

private: System::Void FrequencyCalButton_Click(System::Object *  sender, System::EventArgs *  e)
		 {

			// Internal Crystal Frequency Calibration
			// Starts out with whatever initial value of FG->Error happens to be.
			// This allows iterating to a better error estimate over several
			// frequency cal cycles if desired.

			VNA_RXBUFFER *RxBuf = new VNA_RXBUFFER;
			VNA_TXBUFFER *TxBuf = new VNA_TXBUFFER;
			int Fdesired;

			Fdesired = 100000000;	// output 100 MHz

			TxBuf->TxAccum = FG->DDS(Fdesired);	
			TxBuf->IDAClevelHi = MAX_DDS_LEVEL;		// Max transmit level
			TxBuf->ReplyType = 0;
			TxBuf->MeasureDelay = 0;
			TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level

			VNA->WriteRead(TxBuf, RxBuf);	// write the VNA with CW frequency output

		 }

private: System::Void FreqEnterButton_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 // Get frequency error from the text box and error check it.
			 // If valid, update the frequency error.

			 try
			 {
                 MeasFrequency = Convert::ToInt32(MeasFreqTextBox->Text);
			 }
			 catch (Exception* pe)
			 {
				 MessageBox::Show(pe->Message, "Error");
			 }

			 if((MeasFrequency > 103000000) || (MeasFrequency < 97000000))	// bad data likely
				MessageBox::Show(S"Value appears to be invalid, re-enter",S"Error");
			 else
			 {
				 FrequencyCalStat->Visible = true;
				 FG->Ferror += MeasFrequency - 100000000;
			 }

		 }

private: System::Void DirectivityCalButton_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 // Measure the directivity of the coupler at 21 frequency points 
			 // by using a well-terminated load directly on the TX connector.

			VNA_RXBUFFER *RxBuf = new VNA_RXBUFFER;
			VNA_TXBUFFER *TxBuf = new VNA_TXBUFFER;
			int Fdesired;
			
			TxBuf->ReplyType = 0;
			TxBuf->MeasureDelay = 0;		// Integration by target
			TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level
			TxBuf->IDAClevelHi = MAX_DDS_LEVEL;		// Max transmit level

			progressBar1->Value = 0;

			// run a sweep of 21 frequencies, measuring residual Refl Amplitude
			for (long FreqIdx=0; FreqIdx<21; FreqIdx++)
			{

				// Compute spot frequency

				if (FreqIdx < 9)
					Fdesired = Convert::ToInt32((FreqIdx + 2) * 100000);	// every 100 KHz starting at 200 khz.
				else
					Fdesired = Convert::ToInt32((FreqIdx-8) * 10000000);	// every ten MHz.

				TxBuf->TxAccum = FG->DDS(Fdesired);

				// Take 7 readings, use median value
				for(int k=0; k<7; k++)
				{
					VNA->WriteRead(TxBuf, RxBuf);
					BufferM[k] = RxBuf->ReflMQ;
					BufferI[k] = RxBuf->ReflPI;
					BufferQ[k] = RxBuf->ReflPQ;
				}
				ReflMagPhDir[FreqIdx, 0] = Median7i(BufferM);
				ReflMagPhDir[FreqIdx, 1] = Median7i(BufferI);
				ReflMagPhDir[FreqIdx, 2] = Median7i(BufferQ);

				progressBar1->Value = FreqIdx * progressBar1->Maximum/20;
			}


			Cal->RxDet->DirectivityCal(ReflMagPhDir);		// Save the raw counts in the RxDetector object
			Cal->RxDet->DirCalibrated = true;

			CalStepPass[3] = true;
			DirectivityCalStat->Image = ICO1Label->Image;
			DirectivityCalStat->Visible = true;


		 }

private: System::Void RxAmpButtonOpen_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 // Reflection Amplitude Detector Calibration Open
			 // Captures Magnitude, I-phase, and Q-phase at zero dbm for coupler correction
			 
			VNA_RXBUFFER *RxBuf = new VNA_RXBUFFER;
			VNA_TXBUFFER *TxBuf = new VNA_TXBUFFER;
			int Fdesired;
			

			TxBuf->ReplyType = 0;
			TxBuf->MeasureDelay = 0;			// No Measruement Delay
			TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level

			progressBar1->Value = 0;

			// run a sweep of 21 frequencies, calibrating Refl Amplitude detector
			for (long FreqIdx=0; FreqIdx<21; FreqIdx++)
			{

				// Compute spot frequency

				if (FreqIdx < 9)
					Fdesired = Convert::ToInt32((FreqIdx + 2) * 100000);	// every 100 KHz starting at 200 khz.
				else
					Fdesired = Convert::ToInt32((FreqIdx-8) * 10000000);	// every ten MHz.

				TxBuf->TxAccum = FG->DDS(Fdesired);

 				TxBuf->IDAClevelHi = TxLevLinear(0);	// 0  dbm

				// Take 7  magnitude & phase readings, use median value
				for(int k=0; k<7; k++)
				{
					VNA->WriteRead(TxBuf, RxBuf);
					BufferM[k] = RxBuf->ReflMQ;
					BufferI[k] = RxBuf->ReflPI;
					BufferQ[k] = RxBuf->ReflPQ;
				}
				ReflMagPhOpen[FreqIdx,0] = Median7i(BufferM);
				ReflMagPhOpen[FreqIdx,1] = Median7i(BufferI);
				ReflMagPhOpen[FreqIdx,2] = Median7i(BufferQ);

				progressBar1->Value = FreqIdx * progressBar1->Maximum/20;
				progressBar1->Update();
			}
			CalStepPass[4] = true;
			RxAmpStatOpen->Image = ICO1Label->Image;
			RxAmpStatOpen->Visible = true;

		 }

private: System::Void RxAmpButtonShort_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 // Reflection Amplitude Detector Calibration Shorted
			 // Performs two functions:
			 //		1. Captures amplitude over 60 dB range for linear regression
			 //		2. Captures Mag, I-phase and Q-phase at 0 dbm for coupler correction
			 
			VNA_RXBUFFER *RxBuf = new VNA_RXBUFFER;
			VNA_TXBUFFER *TxBuf = new VNA_TXBUFFER;
			int Fdesired;
			int i;
			long FreqIdx;
			
			double mag, phase;
			double& rmag = mag;
			double& rphase = phase;



			TxBuf->ReplyType = 0;
			TxBuf->MeasureDelay = 0;			// No Measurement Delay
			TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level

			progressBar1->Value = 0;

			// run a sweep of 21 frequencies, calibrating Refl Amplitude detector
			for (FreqIdx=0; FreqIdx<21; FreqIdx++)
			{

				// Compute spot frequency

				if (FreqIdx < 9)
					Fdesired = Convert::ToInt32((FreqIdx + 2) * 100000);	// every 100 KHz starting at 200 khz.
				else
					Fdesired = Convert::ToInt32((FreqIdx-8) * 10000000);	// every ten MHz.

				TxBuf->TxAccum = FG->DDS(Fdesired);

				for (int level=0; level<=59; level++)	// Measure over 60 dB range
				{
 					TxBuf->IDAClevelHi = TxLevLinear(-level);	// 0 to -59 dbm

					// Take 7 readings, use median value
					for(int k=0; k<7; k++)
					{
						VNA->WriteRead(TxBuf, RxBuf);
						BufferM[k] = RxBuf->ReflMQ;
						BufferI[k] = RxBuf->ReflPI;
						BufferQ[k] = RxBuf->ReflPQ;
					}
					ReflMagRegression[FreqIdx, level] = Median7i(BufferM);

					if (level == 0)			// Also record Short Mag & Phase just at the zero dbm level.
					{
						ReflMagPhShort[FreqIdx,0] = Median7i(BufferM);
						ReflMagPhShort[FreqIdx,1] = Median7i(BufferI);
						ReflMagPhShort[FreqIdx,2] = Median7i(BufferQ);
					}
				}
				progressBar1->Value = FreqIdx * progressBar1->Maximum/40;
				progressBar1->Update();
			}

			bool Status = Cal->RxDet->AmpCal(ReflMagRegression);		// Linear regression - sets y = mx + b

			// Collect detailed 1024 point sweep for Shorted Reflection Data

			for (i=0; i<1024; i++)
            {
				// Compute spot frequency

				Fdesired = Convert::ToInt32(200000.0+( (double)i *(120000000.0-200000.0)/1024.0));

				TxBuf->TxAccum = FG->DDS(Fdesired);		
				TxBuf->IDAClevelHi = MAX_DDS_LEVEL;		// Max transmit level
				TxBuf->ReplyType = 0;
				TxBuf->MeasureDelay = 0;
				TxBuf->QDAClevel = QDAC_ZERODBM;	// Reference level

				for (int k=0; k<7; k++)
				{
                    VNA->WriteRead(TxBuf, RxBuf);
					BufferI[k] = RxBuf->ReflPI;
					BufferQ[k] = RxBuf->ReflPQ;
					BufferM[k] = RxBuf->ReflMQ;
				}
				ReflDetailMagPhShort[i, 0] = Median7i(BufferM);
				ReflDetailMagPhShort[i, 1] = Median7i(BufferI);		// filter the raw readings
				ReflDetailMagPhShort[i, 2] = Median7i(BufferQ);

				progressBar1->Value = progressBar1->Maximum/2 + i * progressBar1->Maximum/2048;
				if(i%50 == 0)
                    progressBar1->Update();

			}
			if(Status)
			{
				CalStepPass[5] = true;
				RxAmpStatShort->Image = ICO1Label->Image;
			}
			else
			{
				CalStepPass[5] = false;
				RxAmpStatShort->Image = ICO2Label->Image;
				ShowCalStepFailMessage();
			}
			RxAmpStatShort->Visible = true;
														// Leave coupler correction for 'OK' handler
		 }


private: System::Void Cancelbutton_Click(System::Object *  sender, System::EventArgs *  e)
		 {
			 // Don't save file. Form property tells the caller:   DialogResult:Cancel
		 }

private: System::Void ShowCalStepFailMessage()
		{
			MessageBox::Show(S"This Step returned defective results."
				S"  Check: \n\r\t calibration setup \n\r\t cables \n\r\t adaptors \n\r\t attenuator"
				S"\n\r\t all connections to be sure they are good."
				S"\n\n\rThis step will need to be re-run before finishing calibration."
				S"\n\rClose this window, then 'Cancel' if you do not wish to save Detector Cal data.",
				S"Error: Failed Step", MessageBoxButtons::OK, MessageBoxIcon::Error);
		}
};
}